package de.unibw.i2.mi.oo;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * Class representing interfaces.
 */
public class MIInterface extends MIType {
	private final List<MIInterface> superInterfaces;

	public MIInterface(String name, List<MIInterface> superInterfaces) {
		super(name);
		this.superInterfaces = new ArrayList<MIInterface>(superInterfaces);
	}

	public MIInterface(String name, MIInterface... superInterfaces) {
		this(name, Arrays.asList(superInterfaces));
	}

	public List<MIInterface> getSuperInterfaces() {
		return superInterfaces;
	}

	@Override
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("MIInterface [name=").append(getName());
		if (!getSuperInterfaces().isEmpty()) {
			buf.append(", superInterfaces=[");
			boolean first = true;
			for (MIInterface si : getSuperInterfaces()) {
				if (!first)
					buf.append(", ");
				first = false;
				buf.append(si.getName());
			}
			buf.append("]");
		}
		buf.append("]");
		return buf.toString();
	}

	@Override
	public List<? extends MIType> getDirectSupertypes() {
		return getSuperInterfaces();
	}

	@Override
	public List<Path> getPaths() {
		List<Path> paths = new ArrayList<Path>();
		if (getSuperInterfaces().isEmpty())
			paths.add(new Path());
		else
			for (MIInterface i : getSuperInterfaces())
				paths.addAll(i.getPaths());
		paths.get(0).add(this);
		return paths;
	}

	@Override
	public MIMethod getImplementation(MIMethodSignature method) {
		return null;
	}
}
