package de.unibw.i2.mi.oo;

import java.util.ArrayList;
import java.util.List;

/**
 * A method table of a class. It is either a class method table or an interface
 * method table.
 */
public class MIMethodTable {
	private final int index;
	private final Path path;
	private final List<MIMethod> methods = new ArrayList<MIMethod>();

	/**
	 * Creates a new method table
	 * 
	 * @param aClass
	 *            the class this method table belongs to
	 * @param index
	 *            The index in the list of all method tables of the given class.
	 *            index==0 represents a class method table, index>0 interface
	 *            method tables.
	 * @param path
	 *            the inheritance path for this method table
	 */
	MIMethodTable(MIClass aClass, int index, Path path) {
		super();
		this.index = index;
		this.path = path;
		for (MIMethodSignature s : path.getMethodSignatures())
			methods.add(aClass.getImplementation(s));
	}

	public boolean isClassMethodTable() {
		return index <= 0;
	}

	/**
	 * Returns index in the list of all method tables of this class. This index
	 * can be used to compute the address displacement of an object pointer and
	 * the actual object pointer of the corresponding type. The address
	 * displacement is obtained by multiplying the returned index with the
	 * number of bytes necessary for storing an address, e.g., 4 in a 32bit
	 * architecture.
	 */
	public int getDelta() {
		return index;
	}

	public List<MIMethod> getMethods() {
		return methods;
	}

	public Path getPath() {
		return path;
	}
}
