package de.unibw.i2.mi.oo;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

/**
 * An inheritance path of a type consisting of a sequence of types (classes or
 * interfaces). The first type in the sequence does not have any supertype. If a
 * type B immediately follows a type A in the sequence, then A is the first
 * supertype of b. The first supertype of a class is its superclass, and the
 * first supertype of an interface is the first interface in the list of all of
 * its extended interfaces.
 */
public class Path {
	private final List<MIType> path = new ArrayList<MIType>();

	public void add(MIType t) {
		path.add(t);
	}

	public List<MIType> getPath() {
		return path;
	}

	@Override
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("Path [");
		boolean first = true;
		for (MIType t : path) {
			if (!first)
				buf.append(", ");
			first = false;
			buf.append(t.getName());
		}
		buf.append("]");
		return buf.toString();
	}

	public List<MIMethodSignature> getMethodSignatures() {
		Set<MIMethodSignature> table = new LinkedHashSet<MIMethodSignature>();
		for (MIType t : path) {
			table.addAll(t.getMethodSignatures());
			for (MIType sup : t.getSupertypes())
				table.addAll(sup.getMethodSignatures());
		}
		return new ArrayList<MIMethodSignature>(table);
	}
}
