package de.unibw.i2.mi.oo;


public class Test {

	/**
	 * Creates the example used in "Maschinennahe Programmierung", chapter
	 * "Schnittstellenvererbung mit MI"
	 */
	public static void main(String[] args) {
		MIMethodSignature fSig = new MIMethodSignature("f");
		MIMethodSignature gSig = new MIMethodSignature("g");
		MIMethodSignature hSig = new MIMethodSignature("h");
		MIMethodSignature iSig = new MIMethodSignature("i");
		MIMethodSignature jSig = new MIMethodSignature("j");

		MIInterface i4If = new MIInterface("I4");
		i4If.addMethod(gSig);

		MIInterface i5If = new MIInterface("I5");
		i5If.addMethod(fSig);

		MIInterface i1If = new MIInterface("I1", i5If);
		i1If.addMethod(jSig);

		MIInterface i2If = new MIInterface("I2", i5If, i4If);
		i2If.addMethod(hSig);

		MIInterface i3If = new MIInterface("I3", i4If);
		i3If.addMethod(iSig);

		MIClass aClass = new MIClass("A", false, null);
		aClass.addMethod(new MIMethod(aClass, fSig));
		aClass.addMethod(new MIMethod(aClass, gSig));
		aClass.addMethod(new MIMethod(aClass, hSig));

		MIClass bClass = new MIClass("B", false, aClass, i1If);
		bClass.addMethod(new MIMethod(bClass, fSig));
		bClass.addMethod(new MIMethod(bClass, jSig));

		MIClass cClass = new MIClass("C", false, aClass, i2If, i3If);
		cClass.addMethod(new MIMethod(cClass, gSig));
		cClass.addMethod(new MIMethod(cClass, iSig));

		MIClass dClass = new MIClass("D", false, aClass, i3If);
		dClass.addMethod(new MIMethod(dClass, iSig));

		System.out.println("aClass: " + aClass);
		System.out.println("bClass: " + bClass);
		System.out.println("cClass: " + cClass);
		System.out.println("dClass: " + dClass);
		System.out.println("i1If: " + i1If);
		System.out.println("i2If: " + i2If);
		System.out.println("i3If: " + i3If);
		System.out.println("i4If: " + i4If);
		System.out.println("i5If: " + i5If);
		System.out.print("supertypes of cClass:");
		for (MIType t : cClass.getSupertypes())
			System.out.print(" " + t.getName());
		System.out.println();
		System.out.println();

		methods(aClass);
		methods(bClass);
		methods(cClass);
		methods(dClass);

		System.out.println();
		showUpcastTable(aClass, bClass, cClass, dClass, i1If, i2If, i3If, i4If,
				i5If);
	}

	/**
	 * Prints the method tables of the given class to stdout.
	 */
	private static void methods(MIClass aClass) {
		System.out.printf("%-3s %s\n", aClass.getName() + ":",
				aClass.getPaths());
		int i = 0;
		for (MIMethodTable table : aClass.getMethodTables()) {
			boolean first = true;
			System.out.printf("    Table %2d: ", ++i);
			if (!table.isClassMethodTable()) {
				System.out.printf("delta=%d\n", table.getDelta());
				first = false;
			}
			for (MIMethod m : table.getMethods()) {
				if (!first)
					System.out.print("              ");
				first = false;
				System.out.println(m.shortString());
			}
		}
	}

	/**
	 * prints the upcast table to stdout.
	 * 
	 * @param types
	 *            the sequence of types used as from and to tyes
	 */
	private static void showUpcastTable(MIType... types) {
		System.out.print("    |");
		for (MIType t : types)
			System.out.printf(" %2s ", t.getName());
		System.out.println();
		System.out.print("----+");
		for (int i = 0; i < types.length; i++)
			System.out.printf("----");
		System.out.println();
		for (MIType from : types) {
			System.out.printf("%-4s|", from.getName());
			for (MIType to : types) {
				int diff = from.upcastDiff(to);
				if (diff < 0)
					System.out.print("  - ");
				else
					System.out.printf(" %2d ", 4 * diff);
			}
			System.out.println();
		}
	}
}
