package de.unibw.i2.mi.oo;


public class Test2 {

	/**
	 * Creates the example used in "Maschinennahe Programmierung", chapter
	 * "Schnittstellenvererbung mit MI"
	 */
	public static void main(String[] args) {
		MIMethodSignature geSig = new MIMethodSignature("ge");
		MIMethodSignature llSig = new MIMethodSignature("ll");
		MIMethodSignature sKSig = new MIMethodSignature("sK");
		MIMethodSignature sBSig = new MIMethodSignature("sB");
		MIMethodSignature suSig = new MIMethodSignature("su");

		MIInterface FIf = new MIInterface("F");
		FIf.addMethod(geSig);
		FIf.addMethod(llSig);

		MIInterface SIf = new MIInterface("S", FIf);
		SIf.addMethod(sBSig);

		MIInterface AIf = new MIInterface("A", FIf);
		AIf.addMethod(sKSig);

		MIInterface AFIf = new MIInterface("AF", AIf, SIf);
		AFIf.addMethod(suSig);

		MIClass aiClass = new MIClass("AI", false, null, AIf);
		aiClass.addMethod(new MIMethod(aiClass, geSig));
		aiClass.addMethod(new MIMethod(aiClass, llSig));
		aiClass.addMethod(new MIMethod(aiClass, sKSig));

		MIClass siClass = new MIClass("SI", false, null, SIf);
		siClass.addMethod(new MIMethod(siClass, geSig));
		siClass.addMethod(new MIMethod(siClass, llSig));
		siClass.addMethod(new MIMethod(siClass, sBSig));

		MIClass afiClass = new MIClass("AFI", false, aiClass, AFIf);
		afiClass.addMethod(new MIMethod(afiClass, llSig));
		afiClass.addMethod(new MIMethod(afiClass, sBSig));
		afiClass.addMethod(new MIMethod(afiClass, suSig));

		System.out.println("aiClass: " + aiClass);
		System.out.println("siClass: " + siClass);
		System.out.println("afiClass: " + afiClass);
		System.out.println("FIf: " + FIf);
		System.out.println("SIf: " + SIf);
		System.out.println("AIf: " + AIf);
		System.out.println("AFIf: " + AFIf);
		System.out.print("supertypes of afiClass:");
		for (MIType t : afiClass.getSupertypes())
			System.out.print(" " + t.getName());
		System.out.println();
		System.out.println();

		methods(aiClass);
		methods(siClass);
		methods(afiClass);

		System.out.println();
		showUpcastTable(aiClass, siClass, afiClass, AFIf, SIf, AIf, FIf);
	}

	/**
	 * Prints the method tables of the given class to stdout.
	 */
	private static void methods(MIClass aClass) {
		System.out.printf("%-3s %s\n", aClass.getName() + ":",
				aClass.getPaths());
		int i = 0;
		for (MIMethodTable table : aClass.getMethodTables()) {
			boolean first = true;
			System.out.printf("    Table %2d: ", ++i);
			if (!table.isClassMethodTable()) {
				System.out.printf("delta=%d\n", table.getDelta());
				first = false;
			}
			for (MIMethod m : table.getMethods()) {
				if (!first)
					System.out.print("              ");
				first = false;
				System.out.println(m.shortString());
			}
		}
	}

	/**
	 * prints the upcast table to stdout.
	 * 
	 * @param types
	 *            the sequence of types used as from and to tyes
	 */
	private static void showUpcastTable(MIType... types) {
		System.out.print("    |");
		for (MIType t : types)
			System.out.printf(" %2s ", t.getName());
		System.out.println();
		System.out.print("----+");
		for (int i = 0; i < types.length; i++)
			System.out.printf("----");
		System.out.println();
		for (MIType from : types) {
			System.out.printf("%-4s|", from.getName());
			for (MIType to : types) {
				int diff = from.upcastDiff(to);
				if (diff < 0)
					System.out.print("  - ");
				else
					System.out.printf(" %2d ", 4 * diff);
			}
			System.out.println();
		}
	}
}
